#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_rotozoom.h>

#define SCREEN_WIDTH 1024
#define SCREEN_HEIGHT 768

#define FONT_NAME "Bienvenu.ttf"
#define FONT_SIZE 20

#define BALL_WIDTH 6
#define BALL_HEIGHT 6
#define BALL_SPEED 5

#define PADDLE_WIDTH 8
#define PADDLE_HEIGHT 80
#define PADDLE_SPEED 5

#define POINTS_TO_WIN 3

#define GAME_SPEED 15
#define SLOPE_MAX_DY 2
#define MAX_DY 10
#define MAX_DX 10

#define UPPER_BOUND 0
#define LOWER_BOUND SCREEN_HEIGHT
#define LEFT_BOUND 0
#define RIGHT_BOUND SCREEN_WIDTH

#define SAVER_DX 2
#define SAVER_DY 2

#define SAVER_START 500

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

struct direction {
  int up;
  int down;
  int left;
  int right;
  
  int points;
  
  int stick;
  
  SDL_Rect paddle;
};

struct {
  struct {
    int width;
    int height;
  } screen;
  
  int running;
  int saver;
  
  struct {
    int speed;
    
    int dx;
    int dy;
    
    SDL_Rect rect;
  } ball,saver_state;
  
  struct direction player1;
  struct direction player2;
  
  SDL_Joystick *stick1;
  SDL_Joystick *stick2;
  
} state;

SDL_Surface *screen;
SDL_Event event;
  
SDL_Color color_w = {255,255,255};
SDL_Color color_b = {0,0,0};
  
TTF_Font *font = NULL;

SDL_Surface *text_points_p1 = NULL;
SDL_Surface *text_points_p2 = NULL;

SDL_Rect pos_points_p1,pos_points_p2;

SDL_Surface *text_start = NULL;

SDL_Rect pos_start;

SDL_Surface *wawerko = NULL;

SDL_Rect pos_wawerko;

SDL_Surface *saver = NULL;
SDL_Surface *saver_text = NULL;
SDL_Surface *saver_text2 = NULL;

SDL_Rect pos_saver,pos_saver_text1,pos_saver_text2,pos_saver_wawerko;

SDL_Rect net;
  
int num_joysticks;

Uint32 black;
Uint32 white;

// DEBUG Info

SDL_Surface *text_up;
SDL_Surface *text_down;
SDL_Surface *text_left;
SDL_Surface *text_right;

SDL_Rect pos_up;
SDL_Rect pos_down;
SDL_Rect pos_left;
SDL_Rect pos_right;

SDL_Surface *text_num_joysticks = NULL;
SDL_Surface *text_num_joysticks_num = NULL;
  
SDL_Rect pos_num_joysticks,pos_num_joysticks_num;
  
char num_joysticks_char[2];

// Function defs

int cleanUp( int err );

void resetBall( void );
void genSlope( void );
void constrainBall( void );
void drawBall( void );
int moveBall( void );
void moveSaver( );

void initPaddles( void );
void drawPaddles( void );
void movePaddle(int paddle,int dir);

int initText( void );

void drawPlayfield( void );

int updatePointsText( void );

int initGame( void );

void handleJoyEvent(SDL_Event *event);
void blitSaver( );

int main(int argc,char **argv) {
  
  int ret;
  
  if (SDL_Init( SDL_INIT_VIDEO ) != 0) {
     fprintf(stderr, "Could not initialise SDL: %s\n",SDL_GetError());
     return 1;
  }
  
  if (SDL_InitSubSystem( SDL_INIT_JOYSTICK ) != 0) {
     fprintf(stderr, "Could not initialise SDL: %s\n",SDL_GetError());
     return 1;
  }
  
  if (TTF_Init() == -1) {
     fprintf(stderr,"Could not initialise SDL_ttf: %s\n",TTF_GetError());
     return cleanUp(1);
  }
  
  if ((screen = SDL_SetVideoMode(SCREEN_WIDTH,SCREEN_HEIGHT,8, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN)) == NULL) {
    fprintf(stderr, "Could not set SDL video mode: %s\n",SDL_GetError());
    return cleanUp(1);
  }
  
  state.screen.width = screen->w;
  state.screen.height = screen->h;
  
  num_joysticks = SDL_NumJoysticks();
  
  if (num_joysticks > 0) {
    if (!(state.stick1 = SDL_JoystickOpen(0))) {
      fprintf(stderr,"Could not open joystick 0: %s\n",SDL_GetError());
      return cleanUp(1);
    }
    
    SDL_JoystickEventState(SDL_ENABLE);
  }
  
  if (num_joysticks > 1) {
    if (!(state.stick2 = SDL_JoystickOpen(1))) {
      fprintf(stderr,"Could not open joystick 1: %s\n",SDL_GetError());
      return cleanUp(1);
    }
  }
  
  font = TTF_OpenFont(FONT_NAME,FONT_SIZE);
  if (!font) {
    fprintf(stderr,"Could not open font: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  initText();
  
  if (!(wawerko = SDL_LoadBMP("w.bmp"))) {
    fprintf(stderr,"Could not load image: %s\n",SDL_GetError());
    return cleanUp(1);
  }
  
  if (!(saver = SDL_CreateRGBSurface(0,(saver_text2->w > wawerko->w?saver_text2->w:wawerko->w),saver_text->h + saver_text2->h + wawerko->h + 20,screen->format->BitsPerPixel,screen->format->Rmask,screen->format->Gmask,screen->format->Bmask,screen->format->Amask))) {
    fprintf(stderr,"Could not create Saver Surface: %s\n",SDL_GetError());
    return cleanUp(1);
  }
  
  pos_saver_text1.x = saver->w / 2 - saver_text->w / 2;
  pos_saver_text1.y = 0;
  
  if (SDL_BlitSurface(saver_text,NULL,saver,&pos_saver_text1)) {
    fprintf(stderr,"Could not Blit in Saver Surface: %s\n",SDL_GetError());
    return cleanUp(1);
  }
  
  pos_wawerko.x = (RIGHT_BOUND - LEFT_BOUND) / 2 - wawerko->w / 2;
  pos_wawerko.y = LOWER_BOUND - wawerko->h - 20;
  
  pos_up.x = state.screen.width / 2 - text_up->w / 2;
  pos_up.y = state.screen.height / 2 - text_up->h / 2 - 50;
  
  
  
  pos_down.x = state.screen.width / 2 - text_down->w / 2;
  pos_down.y = state.screen.height / 2 + 50;
  
  
  pos_left.x = state.screen.width / 2 - text_left->w - 50;
  pos_left.y = state.screen.height / 2 - text_left->h / 2;
  
  
  
  pos_right.x = state.screen.width / 2 + 50;
  pos_right.y = state.screen.height / 2 - text_right->h / 2;
  
  
  
  pos_num_joysticks.x = state.screen.width / 2 - text_num_joysticks->w / 2;
  pos_num_joysticks.y = 100;
  
  
  
  pos_num_joysticks_num.x = pos_num_joysticks.x + text_num_joysticks->w + 20;
  pos_num_joysticks_num.y = pos_num_joysticks.y;
  
  SDL_ShowCursor( SDL_DISABLE );
  
  black = SDL_MapRGB(screen->format,0,0,0);
  white = SDL_MapRGB(screen->format,255,255,255);
  
  initPaddles();
  resetBall();
  
  initGame();
  
  state.running = 0;
  
  while (1) {
    SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format,0,0,0));
    
    if (!state.running) {
       while (SDL_PollEvent( &event )) {
	 if (event.type == SDL_QUIT || (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE)) {
	   return cleanUp( 0 );
	 } else if (event.type == SDL_JOYBUTTONDOWN) {
	   if (event.jbutton.state == SDL_PRESSED) {
	     state.player1.stick = event.jbutton.which;
	     state.player2.stick = 1 - state.player1.stick;
	     initGame();
	     state.running = 1;
	   }
	 }
       }
       
       if (state.saver <= SAVER_START) {
        state.saver++;
       }
       drawPlayfield();
    } else {
    
    
    while (SDL_PollEvent( &event )) {
      if (event.type == SDL_QUIT || (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE)) {
	return cleanUp( 0 );
      } else if (event.type == SDL_KEYDOWN) {
	switch (event.key.keysym.sym) {
	  case SDLK_UP:
	    state.player1.up = TRUE;
	    break;
	  default:
	    break;
	}
      } else if (event.type == SDL_KEYUP) {
	switch (event.key.keysym.sym) {
	  case SDLK_UP:
	    state.player1.up = FALSE;
	    break;
	  default: 
	    break;
	}
      } else {
	handleJoyEvent(&event);
      }
    }
    
    
    
    if (state.player1.up) {
      movePaddle(0,1);
    }
    if (state.player1.down) {
      movePaddle(0,0);
    }
    
    if (state.player2.up) {
      movePaddle(1,1);
    }
    if (state.player2.down) {
      movePaddle(1,0);
    }
    
    ret = moveBall();
    
    if (ret == 1) {
      printf("GOOOOAAAALLL Player 1\n");
      state.player1.points++;
      resetBall();
      updatePointsText();
    } else if (ret == 2) {
      printf("GOOOOAAAALLL Player 2\n");
      state.player2.points++;
      resetBall();
      updatePointsText();
    }
    
    //printf("Ball: %d,%d %d,%d\n",state.ball.rect.x,state.ball.rect.y,state.ball.dx,state.ball.dy);
    
    //constrainBall();
    
    drawPlayfield();
    
    /*if (state.player1.up) {
      SDL_BlitSurface(text_up,NULL,screen,&pos_up);
    }
    
    if (state.player1.down) {
      SDL_BlitSurface(text_down,NULL,screen,&pos_down);
    }
    
    if (state.player1.left) {
      SDL_BlitSurface(text_left,NULL,screen,&pos_left);
    }
    
    if (state.player1.right) {
      SDL_BlitSurface(text_right,NULL,screen,&pos_right);
    }*/
    
    drawPaddles();
    drawBall();
    
    //SDL_BlitSurface(text_num_joysticks,NULL,screen,&pos_num_joysticks);
    //SDL_BlitSurface(text_num_joysticks_num,NULL,screen,&pos_num_joysticks_num);
    
    if (state.player1.points >= POINTS_TO_WIN || state.player2.points >= POINTS_TO_WIN) {
      state.running = FALSE;
    }
    
    }
    SDL_Flip(screen);
    
    SDL_Delay( GAME_SPEED );
  }
  
  SDL_FreeSurface(text_up);
  SDL_FreeSurface(text_down);
  SDL_FreeSurface(text_left);
  SDL_FreeSurface(text_right);
  
  SDL_FreeSurface(text_num_joysticks);
  SDL_FreeSurface(text_num_joysticks_num);
  
  if (num_joysticks > 0 && SDL_JoystickOpened(0)) {
    SDL_JoystickClose(state.stick1);
  }
  
  if (num_joysticks > 1 && SDL_JoystickOpened(1)) {
    SDL_JoystickClose(state.stick2);
  }
  
  return cleanUp( 0 );
}

int cleanUp( int err ) {
  TTF_Quit();
  SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
  SDL_Quit();
  
  return err;
}

void resetBall( void ) {
  
  state.ball.rect.w = BALL_WIDTH;
  state.ball.rect.h = BALL_HEIGHT;
  
  state.ball.rect.x = screen->w / 2 - state.ball.rect.w / 2;
  state.ball.rect.y = screen->h / 2 - state.ball.rect.h / 2;
  
  state.ball.speed = BALL_SPEED;
  
  genSlope();
  
}

void moveSaver() {
  state.saver_state.rect.x += state.saver_state.dx;
  state.saver_state.rect.y += state.saver_state.dy;
  
  if (state.saver_state.rect.x <= (LEFT_BOUND + state.saver_state.rect.w / 2) || state.saver_state.rect.x >= (RIGHT_BOUND - state.saver_state.rect.w / 2)) {
    state.saver_state.dx *= -1;
  }
  
  if (state.saver_state.rect.y <= UPPER_BOUND || state.saver_state.rect.y >= (LOWER_BOUND - state.saver_state.rect.h)) {
    state.saver_state.dy *= -1;
  }
}

int moveBall( void ) {
  
  state.ball.rect.x += state.ball.dx;
  state.ball.rect.y += state.ball.dy;
  
  if (state.ball.rect.y <= UPPER_BOUND || state.ball.rect.y >= (LOWER_BOUND - state.ball.rect.h)) {
    state.ball.dy *= -1;
  }
  
  constrainBall();
  //printf("Ball: %d,%d %d,%d\n",state.ball.rect.x,state.ball.rect.y,state.ball.dx,state.ball.dy);
  if (state.ball.rect.x <= LEFT_BOUND + PADDLE_WIDTH) {
    
     // increase speed
    
    if (state.ball.dx > 0) { state.ball.dx += 1; } else { state.ball.dx -= 1; }
    if (state.ball.dy > 0) { state.ball.dy += 1; } else { state.ball.dy -= 1; }
    
    if (state.ball.rect.y + BALL_HEIGHT >= state.player1.paddle.y && state.ball.rect.y <= state.player1.paddle.y + PADDLE_HEIGHT) {
      state.ball.dx *= -1;
      
      if (state.ball.rect.y < state.player1.paddle.y + PADDLE_HEIGHT / 2) {
	if (state.ball.dy < 0) {
	  state.ball.dy -= 1;
	} else {
	  state.ball.dy *= -1;
	}
      } else if (state.ball.rect.y > state.player1.paddle.y + PADDLE_HEIGHT / 2) {
	if (state.ball.dy > 0) {
	  state.ball.dy += 1;
	} else {
	  state.ball.dy *= -1;
	}
      }
      
    } else {
      return 2;
    }
  }
  
  if (state.ball.rect.x + BALL_WIDTH >= RIGHT_BOUND - PADDLE_WIDTH) {
    
     // increase speed
    
    if (state.ball.dx > 0) { state.ball.dx += 1; } else { state.ball.dx -= 1; }
    if (state.ball.dy > 0) { state.ball.dy += 1; } else { state.ball.dy -= 1; }
    
    if (state.ball.rect.y + BALL_HEIGHT >= state.player2.paddle.y && state.ball.rect.y <= state.player2.paddle.y + PADDLE_HEIGHT) {
      state.ball.dx *= -1;
      
      if (state.ball.rect.y < state.player2.paddle.y + PADDLE_HEIGHT / 2) {
	if (state.ball.dy < 0) {
	  state.ball.dy -= 1;
	} else {
	  state.ball.dy *= -1;
	}
      } else if (state.ball.rect.y > state.player2.paddle.y + PADDLE_HEIGHT / 2) {
	if (state.ball.dy > 0) {
	  state.ball.dy += 1;
	} else {
	  state.ball.dy *= -1;
	}
      }
      
    } else {
      return 1;
    }
  }
  
  if (state.ball.dy > MAX_DY) {
    state.ball.dy = MAX_DY;
  } else if (state.ball.dy < -MAX_DY) {
    state.ball.dy = -MAX_DY;
  }
  
  if (state.ball.dx > MAX_DX) {
    state.ball.dx = MAX_DX;
  } else if (state.ball.dx < -MAX_DX) {
    state.ball.dx = -MAX_DX;
  }
  
  return 0;
}

void constrainBall( void ) {
  
  if (state.ball.rect.y < UPPER_BOUND) {
    state.ball.rect.y = UPPER_BOUND;
  }
  
  if (state.ball.rect.y > LOWER_BOUND) {
    state.ball.rect.y = LOWER_BOUND;
  }
  
  if (state.ball.rect.x < LEFT_BOUND) {
    state.ball.rect.x = LEFT_BOUND;
  }
  
  if (state.ball.rect.x > RIGHT_BOUND) {
    state.ball.rect.x = RIGHT_BOUND;
  }
}

void genSlope( void ) {
  srand( time( NULL ) );
  
  state.ball.dy = 1 + (int)( (float) SLOPE_MAX_DY * rand() / (RAND_MAX + 1.0));
  
  state.ball.dx = BALL_SPEED - state.ball.dy;
  
  if ((int)(2.0 * rand() / (RAND_MAX + 1.0))) {
    state.ball.dx *= -1;
  }
  
  if ((int)(2.0 * rand() / (RAND_MAX + 1.0))) {
    state.ball.dy *= -1;
  }
}

void initPaddles( void ) {
  // init left paddle
  
  state.player1.paddle.w = PADDLE_WIDTH;
  state.player1.paddle.h = PADDLE_HEIGHT;
  
  state.player1.paddle.x = LEFT_BOUND;
  state.player1.paddle.y = UPPER_BOUND + ((LOWER_BOUND - UPPER_BOUND)/2) - PADDLE_HEIGHT / 2;
  
  // init right paddle
  
  state.player2.paddle.w = PADDLE_WIDTH;
  state.player2.paddle.h = PADDLE_HEIGHT;
  
  state.player2.paddle.x = RIGHT_BOUND - PADDLE_WIDTH;
  state.player2.paddle.y = UPPER_BOUND + ((LOWER_BOUND - UPPER_BOUND)/2) - PADDLE_HEIGHT / 2;
  
}

void movePaddle(int paddle,int dir) {
  if (paddle == 0) {
    if (dir) {
      state.player1.paddle.y -= PADDLE_SPEED;
    } else {
      state.player1.paddle.y += PADDLE_SPEED;
    }
    
    if (state.player1.paddle.y < UPPER_BOUND) {
      state.player1.paddle.y = UPPER_BOUND;
    } else if (state.player1.paddle.y > LOWER_BOUND - PADDLE_HEIGHT) {
      state.player1.paddle.y = LOWER_BOUND - PADDLE_HEIGHT;
    }
  } else if (paddle == 1) {
    if (dir) {
      state.player2.paddle.y -= PADDLE_SPEED;
    } else {
      state.player2.paddle.y += PADDLE_SPEED;
    }
    
    if (state.player2.paddle.y < UPPER_BOUND) {
      state.player2.paddle.y = UPPER_BOUND;
    } else if (state.player2.paddle.y > LOWER_BOUND - PADDLE_HEIGHT) {
      state.player2.paddle.y = LOWER_BOUND - PADDLE_HEIGHT;
    }
  }
}

void drawPaddles( void ) {
  SDL_FillRect(screen,&state.player1.paddle,white);
  SDL_FillRect(screen,&state.player2.paddle,white);
}

void drawBall( void ) {
  state.ball.rect.w = BALL_WIDTH;
  state.ball.rect.h = BALL_HEIGHT;
  SDL_FillRect(screen,&state.ball.rect,white);
}

int initText( void ) {
  if (!(text_up = TTF_RenderText_Solid(font,"UP",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  if (!(text_down = TTF_RenderText_Solid(font,"DOWN",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  if (!(text_left = TTF_RenderText_Solid(font,"LEFT",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  if (!(text_right = TTF_RenderText_Solid(font,"RIGHT",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  if (!(text_num_joysticks = TTF_RenderText_Solid(font,"Joysticks:",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  num_joysticks_char[0] = '0' + SDL_NumJoysticks();
  num_joysticks_char[1] = 0;
  
  if (!(text_num_joysticks_num = TTF_RenderText_Solid(font,num_joysticks_char,color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  if (!(text_start = TTF_RenderText_Solid(font,"Knopf drcken zum starten",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  text_start = rotozoomSurface(text_start,-90,1.0,0);
  
  pos_start.x = (RIGHT_BOUND - LEFT_BOUND) / 4;
  pos_start.y = (LOWER_BOUND - UPPER_BOUND) / 2 - text_start->h / 2;
  
  if (!(saver_text = TTF_RenderText_Solid(font,"PONG!",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  if (!(saver_text2 = TTF_RenderText_Solid(font,"Knopf drcken zum starten",color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  
}

int updatePointsText( void ) {
  char points_text[4];
  
  points_text[0] = '0' + state.player1.points;
  points_text[1] = ':';
  points_text[2] = '0' + state.player2.points;
  points_text[3] = 0;
  
  if (!(text_points_p1 = TTF_RenderText_Solid(font,points_text,color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  text_points_p1 = rotozoomSurface(text_points_p1,-90,1.0,0);
  
  points_text[0] = '0' + state.player2.points;
  points_text[2] = '0' + state.player1.points;
  
  if (!(text_points_p2 = TTF_RenderText_Solid(font,points_text,color_w))) {
    fprintf(stderr,"Could not render text: %s\n",TTF_GetError());
    return cleanUp(1);
  }
  
  text_points_p2 = rotozoomSurface(text_points_p2,90,1.0,0);
  
  pos_points_p1.x = (RIGHT_BOUND - LEFT_BOUND) / 2 - 50 - text_points_p1->w;
  pos_points_p1.y = UPPER_BOUND + 20;
  
  pos_points_p2.x = (RIGHT_BOUND - LEFT_BOUND) / 2 + 50;
  pos_points_p2.y = UPPER_BOUND + 20;
}

void drawPlayfield( void ) {
  
  if (state.running > 0) {
  
    SDL_BlitSurface(text_points_p1,NULL,screen,&pos_points_p1);
    SDL_BlitSurface(text_points_p2,NULL,screen,&pos_points_p2);
  
    SDL_FillRect(screen, &net,white);
  
    SDL_BlitSurface(wawerko,NULL,screen,&pos_wawerko);
  
  } else {
  
    if (state.saver > SAVER_START) {
      blitSaver();
      moveSaver();
    } else {
      blitSaver();
    }
  }
}

void blitSaver( ) {

  state.saver_state.rect.w = (saver_text2->w > wawerko->w?saver_text2->w:wawerko->w);
  state.saver_state.rect.h = saver_text->h + saver_text2->h + wawerko->h + 10;

  pos_saver_text1.x = state.saver_state.rect.x - saver_text->w / 2;
  pos_saver_text1.y = state.saver_state.rect.y;
  
  SDL_BlitSurface(saver_text,NULL,screen,&pos_saver_text1);
  
  pos_saver_text2.x = state.saver_state.rect.x - saver_text2->w / 2;
  pos_saver_text2.y = state.saver_state.rect.y + saver_text->h + 5;
  
  SDL_BlitSurface(saver_text2,NULL,screen,&pos_saver_text2);
  
  pos_saver_wawerko.x = state.saver_state.rect.x - wawerko->w / 2;
  pos_saver_wawerko.y = pos_saver_text2.y + saver_text2->h + 5;
  
  SDL_BlitSurface(wawerko,NULL,screen,&pos_saver_wawerko);
}

int initGame( void ) {
  
  net.x = (RIGHT_BOUND - LEFT_BOUND) / 2;
  net.y = UPPER_BOUND;
  net.w = 1;
  net.h = (LOWER_BOUND - UPPER_BOUND);
  
  state.player1.points = 0;
  state.player2.points = 0;
  
  state.player1.up = FALSE;
  state.player1.down = FALSE;
  state.player1.left = FALSE;
  state.player1.right = FALSE;
  
  state.player2.up = FALSE;
  state.player2.down = FALSE;
  state.player2.left = FALSE;
  state.player2.right = FALSE;
  
  state.saver = 0;
  
  state.saver_state.dx = SAVER_DX;
  state.saver_state.dy = SAVER_DY;
  
  state.saver_state.rect.x = (RIGHT_BOUND - LEFT_BOUND) / 2;
  state.saver_state.rect.y = (LOWER_BOUND - UPPER_BOUND) / 2 - -saver_text->h - saver_text2->h / 2 - 20;
  
  
  initPaddles();
  resetBall();
  updatePointsText();
}

void handleJoyEvent(SDL_Event *event) {
  if (event->type == SDL_JOYAXISMOTION) {
    if (event->jaxis.which == state.player1.stick) {
      if (event->jaxis.axis == 0) {
	if (event->jaxis.value < 0) {
	  state.player1.right = FALSE;
	  state.player1.left = TRUE;
	} else if (event->jaxis.value == 0) {
	  state.player1.right = FALSE;
	  state.player1.left = FALSE;
	} else if (event->jaxis.value > 0) {
	  state.player1.right = TRUE;
	  state.player1.left = FALSE;
	}
      } else if (event->jaxis.axis == 1) {
	if (event->jaxis.value < 0) {
	  state.player1.up = TRUE;
	  state.player1.down = FALSE;
	} else if (event->jaxis.value == 0) {
	  state.player1.up = FALSE;
	  state.player1.down = FALSE;
	} else if (event->jaxis.value > 0) {
	  state.player1.up = FALSE;
	  state.player1.down = TRUE;
	}
      } 
    } else if (event->jaxis.which == state.player2.stick) {
      if (event->jaxis.axis == 0) {
	if (event->jaxis.value < 0) {
	  state.player2.right = FALSE;
	  state.player2.left = TRUE;
	} else if (event->jaxis.value == 0) {
	  state.player2.right = FALSE;
	  state.player2.left = FALSE;
	} else if (event->jaxis.value > 0) {
	  state.player2.right = TRUE;
	  state.player2.left = FALSE;
	}
      } else if (event->jaxis.axis == 1) {
	if (event->jaxis.value < 0) {
	  state.player2.up = TRUE;
	  state.player2.down = FALSE;
	} else if (event->jaxis.value == 0) {
	  state.player2.up = FALSE;
	  state.player2.down = FALSE;
	} else if (event->jaxis.value > 0) {
	  state.player2.up = FALSE;
	  state.player2.down = TRUE;
	}
      } 
    }
  }
}
